import time
import fpstimer
from playsound import playsound
import cursor
from vid_info import vid_info
from to_ascii import to_ascii
from pydub import AudioSegment
import os
import tempfile

"""ANSI COLOR CODES"""
Red = "\u001b[31m"
Green = "\u001b[32m"
Cyan = "\u001b[36m"
Reset = "\u001b[m"
Reset_cursor = "\u001b[H"

def main(args):

    """Store terminal arguments to variables"""
    scale_num = args.downscale
    colored_frames = args.colored
    play_audio = args.audio
    audio_file_name = "video_audio.mp3"
    chars_list = [args.characters[char] for char in range(len(args.characters))]
    video = args.video_path
    realtime = args.realtime

    try:
        print(f"{Reset_cursor}~~~ Terminal Video Player ~~~\n{Green}Video file detected: {video}.{Reset}")
        video_audio = AudioSegment.from_file(video, "mp4")

    except FileNotFoundError:
        print(f"{Reset_cursor}~~~ Terminal Video Player ~~~\n{Red}Couldn't find video file '{video}'.{Reset}")
        input("Press Enter to Exit")
        exit()

    if play_audio:

        print(f"{Cyan}Generating audio ...{Reset}")
        directory = tempfile.TemporaryDirectory()
        mp3_file_path = os.path.join(directory.name, str(audio_file_name + ".mp3"))
        video_audio.export(mp3_file_path, format="mp3")
        print(f"{Green}Audio Generated!{Reset}")

    cursor.hide()
    frame_info = vid_info(video)
    FPS = frame_info.get_framerate() #Get the video's framerate
    TOTAL_FRAMES = int(frame_info.get_framecount()) #Get the video's total number of frames
    timer = fpstimer.FPSTimer(FPS) #Set the script's framerate to video's framerate
    print("\u001b[2J")

    if realtime:
        input(f"{Reset_cursor}Press Enter to start the playback")
        print("\u001b[2J")

        if play_audio:
            playsound(mp3_file_path, block=False)

        for frame_number in range(TOTAL_FRAMES):
            start = time.perf_counter()

            image = frame_info.get_frame(frame_number)
            frame = to_ascii(image, scale_num, chars_list=chars_list)
                
            if colored_frames:
                frame = frame.asciify_colored()
                
            else:
                frame = frame.asciify()

            print(Reset_cursor + frame + Reset)
            timer.sleep()
            fps = 1 // (time.perf_counter() - start)
            print(f"FPS: {fps}")

    else:

        rendered_result = []
        start = time.time()
        
        for frame_number in range(TOTAL_FRAMES):

            image = frame_info.get_frame(frame_number)
            frame = to_ascii(image, scale_num, chars_list=chars_list)
                
            if colored_frames:
                frame = frame.asciify_colored()
                
            else:
                frame = frame.asciify()

            rendered_result.append(frame)
            print(f"{Reset_cursor}{Cyan}Rendering frames: {frame_number}/{TOTAL_FRAMES} | Elapsed time: {int(time.time() - start)}s{Reset}")

        print(f"{Green}Successfully rendered {TOTAL_FRAMES} frames in {int(time.time() - start)}s.{Reset}")
        input("Press Enter to start the playback")
        print("\u001b[2J")

        if play_audio:
            playsound(mp3_file_path, block=False)

        for frame in rendered_result:
            start = time.perf_counter()
            print(Reset_cursor + frame + Reset)
            timer.sleep()
            fps = 1 // (time.perf_counter() - start)
            print(f"FPS: {fps}")


    cursor.show()
    print(f"{Reset_cursor}{Green}Video Playback finished successfully.{Reset}\u001b[0J")
    input("Press Enter to Exit")
    exit()


"""Make ANSI Escape Codes work on Windows"""
if os.name == "nt":
    os.system("cls")

"""Create terminal arguments"""
import argparse
parser = argparse.ArgumentParser()
parser.add_argument("video_path", help="The video to asciify <path>")
parser.add_argument("downscale", help="How many times to downscale the frames (integer)", type=int)
parser.add_argument("-a", "--audio", help="Play video audio", action="store_true")
parser.add_argument("-c", "--colored", help="Colored output", action="store_true")
parser.add_argument("-chars", "--characters", help="Custom characters(low density to high)", default=[' ', 's', '#', '$','@'])
parser.add_argument("-rt", "--realtime", help="Play video audio", action="store_true")
args = parser.parse_args()

main(args)


from PIL import Image


class to_ascii:
    def __init__(self, file, downscale, width_multiplication = 2.5, chars_list = [' ', '.', "'", '`', '^', ',', ':', ';', 'I', 'l', '!', 'i', '>', '<', '~', '+', '_', '-', '?', ']', '[', '}', '{', '1', ')', '(', '|', '/', 't', 'f', 'j', 'r', 'x', 'n', 'u', 'v', 'c', 'z', 'X', 'Y', 'U', 'J', 'C', 'L', 'Q', '0', 'O', 'Z', 'm', 'w', 'q', 'p', 'd', 'b', 'k', 'h', 'a', 'o', '*', '#', 'M', 'W', '&', '8', '%', 'B', '@', '$']):
        
        if type(file) is str:
            self.frame = Image.open(file)
        
        elif isinstance(file, Image.Image):
            self.frame = file

        else:
            raise Exception("Unsupported File Format")
        
        self.scale = downscale
        self.frame = self.frame.resize((int(self.frame.width//self.scale*width_multiplication), self.frame.height//self.scale))
        self.grayscale_pixels = list(self.frame.convert("L").getdata())
        self.rgb_pixels = list(self.frame.convert('RGB').getdata())
        self.chars_list = chars_list
        self.character_value = len(self.chars_list)/256

    def asciify(self) -> str:
        return "".join([self.chars_list[int(pixel*self.character_value)] if pixel_count % self.frame.width != 0 else "\n" for pixel_count, pixel in enumerate(self.grayscale_pixels)])[1:]

    def asciify_colored(self) -> str:
        return "".join([f"\u001b[38;2;{b};{g};{r}m{self.chars_list[int(self.grayscale_pixels[pixel_count]*self.character_value)]}" if pixel_count % self.frame.width != 0 else "\n" for pixel_count, (r, g, b) in enumerate(self.rgb_pixels)])[1:]

    def colors_list(self) -> list:
        return self.rgb_pixels

    def asciify_colored_test(self) -> str:
        result = ""
        for pixel_count, (r, g, b) in enumerate(self.rgb_pixels):
            if pixel_count % self.frame.width == 0 and pixel_count != 0:
                result += "\n"
            
            result += f"\u001b[38;2;{b};{g};{r}m{self.chars_list[int(self.grayscale_pixels[pixel_count]*self.character_value)]}"
    
        return result


from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()
setup(
    name = 'terminal_video_player_py',
    version = '1.0.2',
    author = '',
    author_email = '',
    license = '',
    description = 'A cli to play videos in the terminal.',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = '',
    py_modules = ['terminal_video_player_py', 'to_ascii', 'vid_info'],
    packages = find_packages(),
    install_requires = [requirements],
    python_requires='>=3.9',
    classifiers=[
    ],
    entry_points = '''
        [console_scripts]
        tvp=terminal_video_player_py:main
    '''
)



import cv2
from PIL import Image

class vid_info:
    def __init__(self, file_name):
        self.vid = cv2.VideoCapture(file_name)

    def get_frame(self, frame_num):
        self.vid.set(1, frame_num)
        success,image = self.vid.read()
        if success:
            img = Image.fromarray(image)
            return img

    def get_framerate(self):

        return self.vid.get(cv2.CAP_PROP_FPS)

    def get_framecount(self):

        return self.vid.get(cv2.CAP_PROP_FRAME_COUNT)